from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Widget import TypesWidget
from Products.Archetypes.Registry import registerWidget

## rr: this has gotten nowhere so far

class DeadlinesWidget(TypesWidget):
    _properties = TypesWidget._properties.copy()
    _properties.update({
        'macro' : "deadlines_widget",
        })
    security = ClassSecurityInfo()

InitializeClass(DeadlinesWidget)

registerWidget(DeadlinesWidget,
               title='Deadlines',
               description="Manage a list of deadlines.",
               ## rr: ?? used_for=('Products.ATExtensions.fields.UrlField',)
               )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerField
from Products.Archetypes.Field import ObjectField
from Products.ATExtensions.datatype.formattablenames \
     import FormattableNames
from Products.ATExtensions.widget import FormattableNamesWidget
from records import RecordsField

class FormattableNamesField(RecordsField):
    """ field for managing 'FormattableNames' (custom type)"""
    _properties = RecordsField._properties.copy()
    _properties.update({
        'type' : 'formattable_names',
        'default' : FormattableNames(),
        'subfields' : ('title','firstname','middlename','lastname',),
        'subfield_sizes' : {'title':10,
                            'firstname': 15,
                            'middlename':15,
                            'lastname':30},
        'widget' : FormattableNamesWidget,
        })
    security = ClassSecurityInfo()
    
    security.declarePrivate('get')
    def get(self, instance, **kwargs):
        value = RecordsField.get(self, instance, **kwargs)
        return FormattableNames(value)

    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        value = self._decode_strings(value, instance, **kwargs)
        value = FormattableNames(value)
        ObjectField.set(self, instance, value, **kwargs)

InitializeClass(FormattableNamesField)

registerField(FormattableNamesField,
              title="Formattable Names",
              description="Used for storing formattable names",
              )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerField

from Products.ATExtensions.Extensions.utils import getDisplayList

from records import RecordsField


class MultipleNamesField(RecordsField):
    """a list of stuctured names (first, last)"""
    _properties = RecordsField._properties.copy()
    _properties.update({
        'type' : 'multiple_names',
        'subfields' : ('first_name','last_name'),
        'subfield_labels':{'first_name':'First or given name(s)',
                           'last_name':'Last or family name(s)',
                           },
        })
    security = ClassSecurityInfo()

InitializeClass(MultipleNamesField)

class PhoneNumbersField(RecordsField):
    """a list of phone numbers (number, type)"""
    _properties = RecordsField._properties.copy()
    _properties.update({
        'type' : 'phone_numbers',
        'subfields' : ('type','number'),
        'subfield_vocabularies':{'type':'PhoneNumberTypes',},
        'innerJoin':': ',
        'outerJoin':'<br />',
        })
    security = ClassSecurityInfo()

    security.declarePublic("PhoneNumberTypes")
    def PhoneNumberTypes(self, instance=None):
        """phone number types"""
        if not instance:
            instance = self
        return getDisplayList(instance, 'phone_number_types')

InitializeClass(PhoneNumbersField)

registerField(MultipleNamesField,
              title="MultipleNames",
              description="Used for storing a list of structured names (first_name, last_name)",
              )

registerField(PhoneNumbersField,
              title="PhoneNumbers",
              description="Used for storing a list of phone numbers (type, number)",
              )




from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Widget import StringWidget
from Products.Archetypes.Registry import registerWidget
from Products.CMFCore.utils import getToolByName
from types import StringType, UnicodeType, StringTypes
from Products.Archetypes.utils import mapply
from zLOG import LOG , WARNING
STRING_TYPES = [StringType, UnicodeType]


class SmartEmailWidget(StringWidget):
    """You should use the SmartEmailWidget only with a SmartEmailField!
    
    The SmartEmailWidget is a StringWidget with the 'mask' property.
    This property holds a string with the name of a method, which will
    be looked up and used to modify the rendering of an email address for
    anonymous view.

    The default value for 'mask' is emailMask. This method is found at 
    ATExtensions/skins/at_extensions/emailMask.py.
    Use this as an example or customize it or define your own method
    to be registered via the widget's 'mask' property.
    """
    _properties = StringWidget._properties.copy()
    _properties.update({'macro_view':"smartemail_widget",
                        'mask':'emailMask'})

    security = ClassSecurityInfo()
    security.declarePublic('EmailMask')
    def EmailMask(self,instance,value=None):
        """ This method calls the masking method as defined in the
	    'emailMask' property of the SmartEmailWidget. If the 
	    method can not be found or is not callable it falls back to
	    using '(et)' as mask for '@'."""
	   
        try :
           val=self.mask
           if type(val) in STRING_TYPES:
                method = getattr(instance, val, None)
                if callable(method):
                    value = method(value)
        except :
           LOG('ATExtensions:', WARNING,
               'There are problems with the emailMask poperty. The script '
               'you state has to be available e.g. in the portal_skins folder.'
               ' It should accept one parameter (the string with the raw '
               'emailaddress) and return a string (the masked emailaddress). '
               'You fall back to standard behavior now.')
           value=value.replace('@','(et)') 
        return value
           
    security.declarePublic('linkToMailForm')      
    def linkToMailForm(self,field,instance):
	"""Constructs a link to the email formular"""
        uid=instance.UID()
        field_id=field
        url=instance.absolute_url()
        link='%s/mailout_form?uid=%s&fid=%s' % (url,uid,field_id)
        return link

InitializeClass(SmartEmailWidget)

registerWidget(SmartEmailWidget,
               title='SmartEmail',
               description="Renders a SmartEmailField.",
               used_for=('Products.ATExtensions.fields.SmartEmailField',)
               )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerField
from Products.Archetypes.Field import ObjectField
from Products.ATExtensions.datatype.formattablename \
     import FormattableName
from Products.ATExtensions.widget import FormattableNameWidget
from record import RecordField

class FormattableNameField(RecordField):
    """ field for managing a 'FormattableName' (custom type)"""
    _properties = RecordField._properties.copy()
    _properties.update({
        'type' : 'formattable_name',
        'default' : FormattableName(),
        'subfields' : ('title','firstname','middlename','lastname',),
        'widget' : FormattableNameWidget,
        })
    security = ClassSecurityInfo()
    
    security.declarePrivate('set')
    def set(self, instance, value, **kwargs):
        value = self._decode_strings(value, instance, **kwargs)
        value = FormattableName(value)
        ObjectField.set(self, instance, value, **kwargs)

InitializeClass(FormattableNameField)

registerField(FormattableNameField,
              title="Formattable Name",
              description="Used for storing a formattable name",
              )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Field import ObjectField
from Products.Archetypes.Registry import registerField

from Products.ATExtensions.widget import UrlWidget

class UrlField(ObjectField):
    """specific field for URLs""" 
    _properties = ObjectField._properties.copy()
    _properties.update({
        'type' : 'url',
        'validators' : ('isPartialUrl'),
        'widget' : UrlWidget,
        })
    security = ClassSecurityInfo()

InitializeClass(UrlField)

registerField(UrlField,
              title="Url",
              description="Used for storing a validated url",
              )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerWidget

class RecordWidget(StringWidget):
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro' : "record_widget",
        'show_hm' : True,             # only meaningful for DateTime subfields
        })
    security = ClassSecurityInfo()

InitializeClass(RecordWidget)

registerWidget(RecordWidget,
               title='Record',
               description="Renders a group of subfields.",
               used_for=('Products.ATExtensions.fields.RecordField',)
               )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerWidget

from Products.ATExtensions.widget import RecordWidget

class RecordsWidget(RecordWidget):
    _properties = RecordWidget._properties.copy()
    _properties.update({
        'macro' : "records_widget",
        })
    security = ClassSecurityInfo()

    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """
        Basic impl for form processing in a widget plus clearing
        of the value if the 'delete all entries' flag is set
        """
        ## check to see if the delete all entries flag was selected
        delete = form.get('%s_delete' % field.getName(), empty_marker)
        if delete is not empty_marker: return {}, {}
        
        value = form.get(field.getName(), empty_marker)
        if value is empty_marker:
            return empty_marker
        if emptyReturnsMarker and value == '':
            return empty_marker

        ## now check to see if we need to delete an individual item
        for idx in range(len(value)-1,-1,-1):
            flag_name = '%s_delete_%s' % (field.getName(),idx)
            delete = form.get(flag_name, None)
            if delete:
                del value[idx]
                del form[flag_name]
        return value, {}    

InitializeClass(RecordsWidget)

registerWidget(RecordsWidget,
               title='Records',
               description="Renders a list of records.",
               used_for=('Products.ATExtensions.fields.RecordsField',)
               )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import RichWidget
from Products.Archetypes.Registry import registerWidget

class RemoteTextWidget(RichWidget):
    _properties = RichWidget._properties.copy()
    _properties.update({
        'macro' : 'remotetext_widget',
        })

    # what to handle now where?
    # customization; triggering upload
    # all in process_form?
    # or do we customize via an extra button with custom action?
    # wouldn't an empty value be sufficient for the non-customized case?
    security = ClassSecurityInfo()

    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """
        Basic impl for form processing in a widget plus setting
        additional values used by our associated field
        """
        value = form.get(field.getName(), empty_marker)
        if value is empty_marker:
            return empty_marker
        if emptyReturnsMarker and value == '':
            return empty_marker

        ## now check to see if we need to take care of some
        ## extra values
        stname = "%s_source_type" % field.getName()
        spname = "%s_source_path" % field.getName()
        stype = form.get(stname, None)
        spath = form.get(spname, None)
        if stype:
            field.setSourceType(instance, stype)
        if spath:
            field.setSourcePath(instance, spath)
        return value, {}    

InitializeClass(RemoteTextWidget)

registerWidget(RemoteTextWidget,
               title="Remote Text Widget",
               description="Populate a field from a remote source",
               used_for="Products.ATExtensions.field.remotetext.RemoteTextField",
               )



## Script (Python) "formatdemo"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=
##title=
##
# This is s special purpose script to illustrate
# the formatting capabilities of the formattable name(s)
#
# it can only be applied to 'Formattable Name Demo' objects
#
n1 = context.getName1()
n2 = context.getName2()
g = context.getFriends()
print n1
print n1("%f %L")
print n2
print n2("%f%l (%F %L)")
print n1+n2
print (n1+n2)(lastsep=' & ')
print g
print (n1+n2+g)(format="%L %f%m", lastsep=' & ')
return printed



# import the individual widgets

from combo import ComboWidget
from datetime import DateTimeWidget
from deadlines import DeadlinesWidget
from record import RecordWidget
from records import RecordsWidget
from url import UrlWidget
from email import EmailWidget 
from formattablename import FormattableNameWidget
from formattablenames import FormattableNamesWidget
from comment import CommentWidget
from remotetext import RemoteTextWidget
from smartemail import SmartEmailWidget
from labeledurl import LabeledUrlWidget



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import SelectionWidget
from Products.Archetypes.Registry import registerWidget


class ComboWidget(SelectionWidget):
    _properties = SelectionWidget._properties.copy()
    _properties.update({
        'type' : 'combobox',
        'macro' : "combo_widget",
        'specify' : False,
        'specifystring' : 'or specify:',
        'br' : True,
        'other' : 'Other',
        'addother' : True,
        })
    security = ClassSecurityInfo()

    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """
        if no value or value equals 'other'
        replace it with the optionally entered one
        """
        value = form.get(field.getName(), empty_marker)
        if value is empty_marker:
            return empty_marker
        if not value or value.lower() in ('other', u'other'):
            # XXX is this generic enough?
            other_name = "%s_other" % field.getName()
            value = form.get(other_name, empty_marker)
        if emptyReturnsMarker and value == '':
            return empty_marker
        return value, {}

InitializeClass(ComboWidget)

registerWidget(ComboWidget,
               title='ComboBox',
               description="Select box together with a string field.",
               used_for=('Products.Archetypes.Field.StringField',)
               )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.Registry import registerPropertyType

class EmailWidget(StringWidget):
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro_view' : "email_widget",
        'at_mask' : '(at)',
        })
    security = ClassSecurityInfo()

InitializeClass(EmailWidget)

registerWidget(EmailWidget,
               title='Email',
               description="Renders an email address (masked for anonymous).",
               used_for=('Products.ATExtensions.fields.EmailField',)
               )
registerPropertyType('at_mask', 'string', EmailWidget)



## Script (Python) "getDisplayView"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=value, join=', '
##title=
##
if same_type(value, '') or same_type(value, u''):
    return value
try:
    return join.join(value)
except TypeError:
    return value



from types import ListType, TupleType, ClassType, FileType, DictType, IntType
from types import StringType, UnicodeType, StringTypes

from App.class_init import InitializeClass
from DateTime import DateTime
from AccessControl import ClassSecurityInfo
from Products.PythonScripts.standard import html_quote
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.Expression import Expression
from Products.CMFCore.Expression import createExprContext
from Products.Archetypes.debug import log
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.Field import ObjectField
from Products.Archetypes.Field import encode, decode
from Products.Archetypes.Registry import registerField, registerPropertyType

# we have to define our own validation handling
try:
    from Products.validation import ValidationChain
    from Products.validation import UnknowValidatorError
    from Products.validation import FalseValidatorError
    from Products.validation.interfaces.IValidator \
         import IValidator, IValidationChain
    HAS_VALIDATION_CHAIN = 1
except ImportError:
    HAS_VALIDATION_CHAIN = 0


from Products.ATExtensions.Extensions.utils import getDisplayList
from Products.ATExtensions.widget import RecordWidget


def providedBy(interface, obj):
    if getattr(interface, 'providedBy', None):
        return interface.providedBy(obj)
    return interface.isImplementedBy(obj)


class RecordField(ObjectField):
    """A field that stores a 'record' (dictionary-like) construct"""
    _properties = ObjectField._properties.copy()
    _properties.update({
        'type' : 'record',
        'default' : {},
        'subfields' : (),
        'subfield_types' : {},
        'subfield_vocabularies' : {},
        'subfield_labels' : {},
        'subfield_sizes' : {},
        'subfield_maxlength' : {},
        'required_subfields': (),
        'subfield_validators': {},
        'subfield_conditions': {},
        'innerJoin' : ', ',
        'outerJoin' : ', ',
        'widget' : RecordWidget,
        })

    security = ClassSecurityInfo()
    
    security.declarePublic('getSubfields') 
    def getSubfields(self):
        """the tuple of sub-fields"""
        return self.subfields

    security.declarePublic('getSubfieldType') 
    def getSubfieldType(self, subfield):
        """
        optional type declaration
        default: string
        """
        return self.subfield_types.get(subfield, 'string')

    security.declarePublic('getSubfieldLabel') 
    def getSubfieldLabel(self, subfield):
        """
        optional custom label for the subfield
        default: the id of the subfield
        """
        return self.subfield_labels.get(subfield, subfield.capitalize())

    def getSubfieldSize(self, subfield, default=40):
        """
        optional custom size for the subfield
        default: 40
        only effective for string type subfields
        """
        return self.subfield_sizes.get(subfield, default)

    def getSubfieldMaxlength(self, subfield):
        """
        otional custom maxlength size for the subfield
        only effective for string type subfields
        """
        return self.subfield_maxlength.get(subfield, 40)

    def isRequired(self,subfield):
        """
        looks whether subfield is included in the list of required subfields
        """
        return subfield in self.required_subfields

    def isSelection(self,subfield):
        """select box needed?"""
        
        return self.subfield_vocabularies.has_key(subfield)

    security.declarePublic('testSubfieldCondition') 
    def testSubfieldCondition(self, subfield, folder, portal, object):
        """Test the subfield condition."""
        try:
            condition = self.subfield_conditions.get(subfield, None)
            if condition is not None:
                __traceback_info__ = (folder, portal, object, condition)
                ec = createExprContext(folder, portal, object)
                return Expression(condition)(ec)
            else:
                return True
        except AttributeError:
            return True

    def getVocabularyFor(self, subfield, instance=None):
        """the vocabulary (DisplayList) for the subfield"""
        ## XXX rr: couldn't we just rely on the field's
        ## Vocabulary method here?
        value = None
        vocab = self.subfield_vocabularies.get(subfield, None)
        if not vocab:
            raise AttributeError, 'no vocabulary found for %s' %subfield

        if isinstance(vocab, DisplayList):
            return vocab
    
        if type(vocab) in StringTypes:
            value = None
            method = getattr(self, vocab, None)
            if method and callable(method):
                value = method(instance)
            else:
                if instance is not None:
                    method = getattr(instance, vocab, None)
                    if method and callable(method):
                        value = method()
            if not isinstance(value, DisplayList):
                raise TypeError, '%s is not a DisplayList %s' %(value, subfield)
            return value
        
        raise TypeError, '%s niether a StringType or a DisplayList for %s' %(vocab, subfield)

    def getViewFor(self, instance, subfield, joinWith=', '):
        """
        formatted value of the subfield for display
        """
        raw = self.getRaw(instance).get(subfield,'')
        if type(raw) in (type(()), type([])):
            raw = joinWith.join(raw)
        # Prevent XSS attacks by quoting all user input
        raw = html_quote(str(raw))
        # this is now very specific
        if subfield == 'email':
            return self.hideEmail(raw,instance)
        if subfield == 'phone':
            return self.labelPhone(raw)
        if subfield == 'fax':
            return self.labelFax(raw)
        if subfield == 'homepage':
            return '<a href="%s">%s</a>' % (raw, raw)
        return raw

    def getSubfieldViews(self,instance,joinWith=', '):
        """
        list of subfield views for non-empty subfields
        """
        result = []
        for subfield in self.getSubfields():
            view = self.getViewFor(instance,subfield,joinWith)
            if view:
                result.append(view)
        return result

    # this is really special purpose and in no ways generic
    def hideEmail(self,email='',instance=None):
	masked = 'email: ' + \
                 email.replace('@', ' (at) ').replace('.', ' (dot) ')
	membertool = getToolByName(instance,'portal_membership',None)
	if membertool is None or membertool.isAnonymousUser():
	    return masked
	return "<a href='mailto:%s'>%s</a>" % (email,email)

    def labelPhone(self,phone=''):
        return 'phone: ' + phone

    def labelFax(self,fax=''):
        return 'fax: ' + fax

    # enable also a string representation of a dictionary
    # to be passed in (external edit may need this)
    # store string values as unicode

    def set(self, instance, value, **kwargs):
        if type(value) in StringTypes:
            try:
                value = eval(value)
                # more checks to add?
            except: # what to catch here?
                pass
        value = self._to_dict(value)
        value = self._decode_strings(value, instance, **kwargs)
        ObjectField.set(self, instance, value, **kwargs)

    def _to_dict(self, value):
        if type(value) != type({}) and hasattr(value, 'keys'):
            new_value = {}
            new_value.update(value)
            return new_value
        return value

    def _decode_strings(self, value, instance, **kwargs):
        new_value = value
        for k, v in value.items():
            if type(v) is type(''):
                nv =  decode(v, instance, **kwargs)
                try:
                    new_value[k] = nv
                except AttributeError: # Records don't provide __setitem__
                    setattr(new_value, k , nv)

            # convert datetimes
            if self.subfield_types.get(k, None) == 'datetime':
                try:
                    val = DateTime(v)
                except: 
                    val = None
                    
                new_value[k] = val

        return new_value

    # Return strings using the site's encoding

    def get(self, instance, **kwargs):
        value = ObjectField.get(self, instance, **kwargs)
        return self._encode_strings(value, instance, **kwargs)

    def _encode_strings(self, value, instance, **kwargs):
        new_value = value
        for k, v in value.items():
            if type(v) is type(u''):
                nv = encode(v, instance, **kwargs)
                try:
                    new_value[k] = nv
                except AttributeError: # Records don't provide __setitem__
                    setattr(new_value, k , nv)
        return new_value

    if HAS_VALIDATION_CHAIN:
        def _validationLayer(self):
            """
            Resolve that each validator is in the service. If validator is
            not, log a warning.

            We could replace strings with class refs and keep things impl
            the ivalidator in the list.

            Note: XXX this is not compat with aq_ things like scripts with __call__
            """
            for subfield in self.getSubfields():
                self.subfield_validators[subfield] = self._subfieldValidationLayer(subfield)
        
        def _subfieldValidationLayer(self, subfield):
            """
            for the individual subfields
            """
            chainname = 'Validator_%s_%s' % (self.getName(), subfield)
            current_validators = self.subfield_validators.get(subfield, ())

            if type(current_validators) is DictType:
                raise NotImplementedError, 'Please use the new syntax with validation chains'
            elif providedBy(IValidationChain, current_validators):
                validators = current_validators
            elif providedBy(IValidator, current_validators):
                validators = ValidationChain(chainname, validators=current_validators)
            elif type(current_validators) in (TupleType, ListType, StringType):
                if len(current_validators):
                    # got a non empty list or string - create a chain
                    try:
                        validators = ValidationChain(chainname, validators=current_validators)
                    except (UnknowValidatorError, FalseValidatorError), msg:
                        log("WARNING: Disabling validation for %s/%s: %s" % (self.getName(), subfield, msg))
                        validators = ()
                else:
                    validators = ()
            else:
                log('WARNING: Unknow validation %s. Disabling!' % current_validators)
                validators = ()

            if not subfield in self.required_subfields:
                if validators == ():
                    validators = ValidationChain(chainname)
                if len(validators):
                    # insert isEmpty validator at position 0 if first validator
                    # is not isEmpty
                    if not validators[0][0].name == 'isEmpty':
                        validators.insertSufficient('isEmpty')
                else:
                    validators.insertSufficient('isEmpty')

            return validators

        security.declarePublic('validate')
        def validate(self, value, instance, errors={}, **kwargs):
            """
            Validate passed-in value using all subfield validators.
            Return None if all validations pass; otherwise, return failed
            result returned by validator
            """
            name = self.getName()
            if errors and errors.has_key(name):
                return True

            if self.required_subfields:
                for subfield in self.required_subfields:
                    sub_value = value.get(subfield, None)
                    res = self.validate_required(instance, sub_value, errors)
                    if res is not None:
                        return res

            # not touched yet
            if self.enforceVocabulary:
                res = self.validate_vocabulary(instance, value, errors)
                if res is not None:
                    return res

            # can this part stay like it is?
            res = instance.validate_field(name, value, errors)
            if res is not None:
                return res

            # this should work
            if self.subfield_validators:
                res = self.validate_validators(value, instance, errors, **kwargs)
                if res is not True:
                    return res

            # all ok
            return None

        def validate_validators(self, value, instance, errors, **kwargs):
            result = True
            total = ''
            for subfield in self.getSubfields():
                subfield_validators = self.subfield_validators.get(subfield, None)
                if subfield_validators:
                    result = subfield_validators(value.get(subfield),
                                                 instance=instance,
                                                 errors=errors,
                                                 field=self,
                                                 **kwargs
                                                 )
                if result is not True:
                    total += result
            return total or result


InitializeClass(RecordField)

registerField(RecordField,
              title="Record",
              description="Used for storing a 'record' (dictionary-like) construct",
              )

registerPropertyType('subfields', 'lines', RecordField)
registerPropertyType('required_subfields', 'lines', RecordField)
registerPropertyType('subfield_validators', 'mapping', RecordField)
registerPropertyType('subfield_types', 'mapping', RecordField)
registerPropertyType('subfield_vocabularies', 'mapping', RecordField)
registerPropertyType('subfield_labels', 'mapping', RecordField)
registerPropertyType('subfield_sizes', 'mapping', RecordField)
registerPropertyType('subfield_maxlength', 'mapping', RecordField)
registerPropertyType('innerJoin', 'string', RecordField)
registerPropertyType('outerJoin', 'string', RecordField)




##########################################################################
#                                                                        #
#           copyright (c) 2004 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""package installer"""

from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from Products.Archetypes.public import process_types, listTypes
from config import *

# the custom validator
from Products.validation import validation
from validator.isPartialUrl import PartialUrlValidator
validation.register(PartialUrlValidator('isPartialUrl'))

# the demo content types
from examples import demo, formattablename

# backwards compatibility
import sys
import ateapi
sys.modules['Products.ATExtensions.at_extensions'] = ateapi

registerDirectory(SKINS_DIR, GLOBALS)

def initialize(context):
    """Initializer called when used as a Zope 2 product."""
    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)
    utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)




from App.class_init import InitializeClass
from Products.Archetypes.Registry import registerField
from Products.ATExtensions.Extensions.utils import getDisplayList
from Products.Archetypes.Field import StringField
from Products.ATExtensions.widget import SmartEmailWidget


class SmartEmailField(StringField):
    """ SmartEmailField with isEmail validator and the SmartEmailWidget """
    _properties = StringField._properties.copy()
    _properties.update({
        'type':'smartemail',
        'validators':('isEmail'),
        'widget':SmartEmailWidget,
        })

    def getRawValue(self,instance):
	    return self.get(instance)
    
InitializeClass(SmartEmailField)


from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerWidget

class CommentWidget(StringWidget):
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro' : "comment_widget",
        'i18n_domain': "plone",
        'visible' : {'edit' : 'visible',
                     'view' : 'invisible',
                     }
        })
    security = ClassSecurityInfo()
    
InitializeClass(CommentWidget)

registerWidget(CommentWidget,
               title='Comment',
               description="Renders a comment in 'base_edit'.",
               used_for=('Products.ATExtensions.fields.CommentField',)
               )



##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

from string import punctuation
from Products.CMFCore.utils import getToolByName
filtered_punctuation = [c for c in punctuation \
                        if c not in "()[]{}<>"]

default_format="%T %F %M %P %L %S"

default_formatter = {'T':'title',
                     'F':'firstname',
                     'M':'middlename',
                     'P':'prefix',
                     'L':'lastname',
                     'S':'suffix',
                     'H':'homepage',
                     'E':'email',
                     }

default_abbreviations = {}


def abbreviate(value, abbr=None, join_abbr=''):
    """
    return an abbreviation for 'value'

    1. from the abbreviations mapping
    2. or the first letter (or letters if value is multivalued)
    3. return '' if value is empty
    """
    if not value:
        return ''
    abbreviations = abbr or default_abbreviations
    special =  abbreviations.get(value, None)
    if special is not None:
        return special
    values = [v[0] for v in value.split()]
    return join_abbr.join(values)


class FormattableName:

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, mapping={}):
        """sets defaults and updates from kw"""
        self._keys = ()
        self.update(mapping)

    def __call__(self, *args, **kwargs):
        value = self.stringformat(*args, **kwargs)
        if kwargs.get('withURL', False):
            homepage = self.getHomePage(*args, **kwargs)
            if homepage:
                value = '<a href="%s">%s</a>' % (homepage, value)
        return value

    def getHomePage(self, context=None, methodname='', *args, **kwargs):
        if self.get('homepage'):
            return self.get('homepage')
        membertool = getToolByName(context, 'portal_membership', None)

        # look up homepage from user
        if self.get('username', False):
            try:
                return membertool.getHomeUrl(self.get('username', ''), 1)
            except ValueError:
                pass
        try:
            return getattr(context, methodname)(self)
        except AttributeError:
            return None

    def __str__(self):
        return self.stringformat()

    def __repr__(self):
        return str(dict(self.items()))

    def __add__(self, other):
        from formattablenames import FormattableNames
        if isinstance(other, FormattableName):
            return FormattableNames([self, other])
        if isinstance(other, FormattableNames):
            return FormattableNames([self]) + other

    def stringformat(self,
                     format=default_format,
                     formatter=default_formatter,
                     abbr=None,
                     join_abbr='',
                     abbrev=None,        # backwards compat
                     lastnamefirst=None, # backwards compat
                     **kw
                     ):
        """Takes a format string to control the rendering
        of the name (inspired by DateTime).

        Default format is '%T %F %M %P %L %S' where the
        keys mapped to by the default formatter are:

           %T -> title: for a leading title (e.g. academic)
           %F -> firstname: for the first first (or given) name
           %M -> middlename: for the middle name(s)
           %P -> prefix: for a last name prefix (e.g. a title of nobility)
           %L -> lastname: for the last name
           %S -> suffix: for a suffix (e.g. a trailing 'PhD' or 'SA')

        Use lowercase keys to get the abbreviated value
        (default: first letter; use the 'abbr' mapping
        to define exceptions; for multi-worded values the first letter
        of each word, joined by the value of 'join_abbr', is returned)

        Pass in a mapping to 'formatter' for custom format key mapping.

        Usage of 'abbrev' and 'lastnamefirst' is deprecated and for
        backwards compatibility only.

        See the test suite for example usages.
        """
        if lastnamefirst:
            format = "L, %F, %P"
        if abbrev: format = format.replace("%F", "%f")

        for key in formatter.keys():
            match = "%"+key
            value = self[formatter[key]]
            format = format.replace(match, value)
            if match.lower() in format:
                format = format.replace(match.lower(),
                                        abbreviate(value, abbr, join_abbr))

        return self._normalize(format)

    def _normalize(self, format):
        """Remove leading, stray, and trailing punctuation
        and collapse whitespece"""
        format = self._remove_punctuation(format.strip())
        for match in ['  ', '()', '[]', '{}', '<>']:
            while match in format:
                format = format.replace(match,' ')
        return format.strip()

    def _remove_punctuation(self, format):
        for c in filtered_punctuation:
            match = ' ' + c + ' '
            format = format.replace(match, ' ')
            while format.startswith(c):
                format = format[1:]
            while format.endswith(c):
                format = format[:-1]
        return format


    # behave like a dict

    def __getitem__(self, key, default=''):
        return getattr(self, key, default)

    def __setitem__(self, key, value):
        setattr(self, key, value)
        if key not in self._keys:
            self._keys += (key,)

    def get(self, key, default=''):
        return getattr(self, key, default) or default

    def update(self, kw=None):
        if not kw:
            return None
        for k,v in kw.items():
            self[k] = v

    def keys(self):
        return self._keys

    def items(self):
        return [(k, self[k]) for k in self._keys]

    # our one hard-coded exception

    def __setattr__(self, name, value):
        if value and name == 'firstnames':
            self['firstname'] = value.split()[0]
            self['middlename'] = ' '.join(value.split()[1:]).strip()
        else:
            self.__dict__[name] = value

    def __getattr__(self, name):
        if name == 'firstnames':
            value = ' '.join([self['firstname'], self['middlename']])
            return value.strip()
        else:
            raise AttributeError





##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

from formattablename import FormattableName

class FormattableNames:
    def __init__(self,names=[]):
        self.names = ()
        for name in names:
            self.append(name)

    def __call__(self, *args, **kwargs):
        return self.strfmt(*args, **kwargs)

    def __getitem__(self,key):
        return self.names[key]

    def __len__(self):
        return len(self.names)

    def append(self,name):
        self.names += (FormattableName(name),)

    def __add__(self, other):
        if isinstance(other, FormattableNames):
            return FormattableNames(self.names + other.names)
        sum = FormattableNames(self.names)
        sum.append(other)
        return sum

    def __str__(self):
        return self()

    def __repr__(self):
        return str(self.names)

    def strfmt(self, *args, **kw):
        length = len(self)
        empty_marker = kw.get('empty_marker', "No names specified")
        sep = kw.get('sep', ', ')
        lastsep = kw.get('lastsep', ', and ')

        if length == 0:
            return empty_marker
        if length == 1:
            return self.names[0](**kw)
        if length == 2:
            if ',' in lastsep:
                lastsep = lastsep.replace(',','')
            return lastsep.join([a(**kw) for a in self.names])

        return sep.join([a(**kw) for a in self.names[:-1]]) \
               + lastsep + self.names[-1](**kw)





from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import StringWidget
from Products.Archetypes.Registry import registerWidget
from Products.Archetypes.Registry import registerPropertyType

class UrlWidget(StringWidget):
    _properties = StringWidget._properties.copy()
    _properties.update({
        'macro' : "url_widget",
        'default_protocol' : 'http',
        })
    security = ClassSecurityInfo()
    
    security.declarePublic('process_form')
    def process_form(self, instance, field, form, empty_marker=None,
                     emptyReturnsMarker=False):
        """
        add the default protocol if there is none
        """
        value = form.get(field.getName(), empty_marker)
        if value is empty_marker:
            return empty_marker
        if emptyReturnsMarker and value == '':
            return empty_marker
        if value and '://' not in value:
            value = self.default_protocol + '://' + value
        return value, {}

InitializeClass(UrlWidget)

registerWidget(UrlWidget,
               title='Url',
               description="Renders a URL in an anchor tag.",
               used_for=('Products.ATExtensions.fields.UrlField',)
               )
registerPropertyType('default_protocol', 'string', UrlWidget)



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerField

from Products.ATExtensions.Extensions.utils import getDisplayList
from Products.ATExtensions.widget import LabeledUrlWidget

from record import RecordField

class AddressField(RecordField):
    """ dedicated address field"""
    _properties = RecordField._properties.copy()
    _properties.update({
        'type' : 'address',
        'subfields' : ('street1','street2','zip','city', 'country'),
        'required_subfields': ('city', 'country'),
        'subfield_labels':{'zip':'ZIP code'},
        'subfield_vocabularies' :{'country':'CountryNames'},
        'outerJoin':'<br />',
        })
    security = ClassSecurityInfo()
    
    security.declarePublic("CountryNames")
    def CountryNames(self, instance=None):
        if not instance:
            instance = self
        return getDisplayList(instance, 'country_names')

InitializeClass(AddressField)

class LocationField(RecordField):
    """ dedicated location (city, country) field"""
    _properties = RecordField._properties.copy()
    _properties.update({
        'type' : 'location',
        'subfields' : ('city', 'country'),
        'subfield_types' : {'country' : 'selection'},
        'subfield_vocabularies' :{'country':'CountryNames'},
        })
    security = ClassSecurityInfo()
    
    security.declarePublic("CountryNames")
    def CountryNames(self, instance=None):
        if not instance:
            instance = self
        return getDisplayList(instance, 'country_names')

InitializeClass(LocationField)

class ContactField(RecordField):
    """name, phone, fax, email, address field"""
    _properties = RecordField._properties.copy()
    _properties.update({
        'type' : 'contact',
        'subfields' : ('name','phone','fax','email', 'address'),
        'innerJoin' : '<br>',
        'subfield_types' : {'address': 'lines'},
        'subfield_maxlength' : {'email' : 256},
        'outerJoin':'<br />',
        })    
    security = ClassSecurityInfo()

InitializeClass(ContactField)

class SimpleNameField(RecordField):
    """just first and last name"""
    _properties = RecordField._properties.copy()
    _properties.update({
        'type' : 'simple_name',
        'subfields' : ('first_name','last_name'),
        'subfield_labels':{'first_name':'First or given name(s)',
                           'last_name':'Last or family name(s)',
                           },
        'outerJoin':' ',
        })    
    security = ClassSecurityInfo()

InitializeClass(SimpleNameField)    

class NetAddressField(RecordField):
    """ dedicated net address field"""
    _properties = RecordField._properties.copy()
    _properties.update({
        'type' : 'net_address',
        'subfields' : ('email','homepage'),
        'subfield_validators':{'homepage':'isURL'},
        'outerJoin':'<br />',
        })
    security = ClassSecurityInfo()

InitializeClass(NetAddressField)

class LabeledUrlField(RecordField):
    """field for holding a link to be displayed using a label"""
    _properties = RecordField._properties.copy()
    _properties.update({
        'type' : 'labeled_url',
        'subfields' : ('label','url'),
        'subfield_validators':{'url':'isURL'},
        'outerJoin':': ',
        'widget' : LabeledUrlWidget,
        })
    security = ClassSecurityInfo()

InitializeClass(LabeledUrlField)


registerField(SimpleNameField,
              title="SimpleName",
              description="Used for storing a structured (first, last) name",
              )

registerField(LocationField,
              title="Location",
              description="Used for storing a location (city, country)",
              )

registerField(AddressField,
              title="Address",
              description="Used for storing an address (street1, street2, zip, city, country)",
              )

registerField(ContactField,
              title="Contact",
              description="Used for storing contact information (name, phone, fax, email, address)",
              )

registerField(NetAddressField,
              title="NetAddress",
              description="Used for storing email and homepage",
              )

registerField(LabeledUrlField,
              title="labeledUrl",
              description="Used for storing a label and a URL",
              )



# the datatypes
from Products.ATExtensions.datatype import *
# the fields
from Products.ATExtensions.field import *
# the widgets
from Products.ATExtensions.widget import *
# the utilities
from Products.ATExtensions.Extensions.utils import getDisplayList



# -*- coding: UTF-8 -*-

##################################################
#                                                #
#    Copyright (C), 2004, Thomas Förster         #
#    <t.foerster@biologie.hu-berlin.de>          #
#                                                #
#    Humboldt University of Berlin               #
#                                                #
##################################################

from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import DateTimeField as BaseField
from Products.Archetypes.Registry import registerField, \
     registerPropertyType

from Products.ATExtensions.widget.datetime import DateTimeWidget

class DateTimeField(BaseField):
    """
    An improved DateTime Field. It allows to specify
    whether only dates or only times are interesting.
    """

    _properties = BaseField._properties.copy()
    _properties.update({
        'type' : 'datetime_ng',
        'widget' : DateTimeWidget,
        'with_time' : 1, # set to False if you want date only objects
        'with_date' : 1, # set to False if you want time only objects
        })
    security = ClassSecurityInfo()

InitializeClass(DateTimeField)

registerField(DateTimeField,
              title="DateTime Field",
              description="An improved DateTimeField, which also allows time or date only specifications.",
              )

registerPropertyType('with_time', 'boolean', DateTimeField)
registerPropertyType('with_date', 'boolean', DateTimeField)



# import the individual fields

from datetime import DateTimeField
from url import UrlField
from email import EmailField
from record import RecordField
from records import RecordsField
from record_examples import SimpleNameField, LocationField,\
     AddressField, ContactField, NetAddressField, LabeledUrlField
from records_examples import MultipleNamesField, PhoneNumbersField
from formattablename import FormattableNameField
from formattablenames import FormattableNamesField
from comment import CommentField
from remotetext import RemoteTextField
from smartemail import SmartEmailField



# -*- coding: UTF-8 -*-

##################################################
#                                                #
#    Copyright (C), 2004, Thomas Förster         #
#    <t.foerster@biologie.hu-berlin.de>          #
#                                                #
#    Humboldt University of Berlin               #
#                                                #
##################################################

from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import CalendarWidget
from Products.Archetypes.Registry import registerWidget

class DateTimeWidget(CalendarWidget):
    _properties = CalendarWidget._properties.copy()
    _properties.update({
        'macro' : "datetime_widget",
        })

    security = ClassSecurityInfo()

InitializeClass(DateTimeWidget)

registerWidget(DateTimeWidget,
               title='DateTime',
               description="An improved DateTime Widget.",
               used_for=('Products.ATExtensions.fields.DateTimeField',)
               )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.PythonScripts.standard import html_quote
from Products.Archetypes.Registry import registerField, registerPropertyType
from Products.ATExtensions.config import HAS_VALIDATION_CHAIN
from Products.ATExtensions.field import RecordField
from Products.ATExtensions.widget import RecordsWidget


class RecordsField(RecordField):
    """A field that stores a 'record' (dictionary-like) construct"""
    _properties = RecordField._properties.copy()
    _properties.update({
        'type' : 'records',
        'default' : [],
        'fixedSize': 0,
        'minimalSize': 0,
        'maximalSize': 9999,
        'innerJoin' : ' ',
        'outerJoin' : ', ',
        'widget' : RecordsWidget,
        })

    security = ClassSecurityInfo()

    def getSize(self, instance):
        """number of records to store"""
        return len(self.getRaw(instance))

    def isSizeFixed(self):
        """do we need an additional line of entry?"""
        return self.fixedSize

    def showMore(self, values):
        """
        return True if the 'More' button should be shown
        False otherwise
        """
        data_length = len(values)
        if data_length < self.maximalSize and not self.fixedSize:
            return True
        else:
            return False

    def getEditSize(self, instance):
        """
        number of record entries to offer in the form
        at least 'minimalSize' or length of the current
        list of records (+1 if 'more' is enabled)
        """
        data_length = len(self.getRaw(instance))
        minimum = max(self.minimalSize, data_length)
        if minimum >= self.maximalSize:
            return minimum  # not to lose data if uploads had added more
        if not self.fixedSize:
            return minimum + 1
        return minimum

    def getSubfieldValue(self, values, idx, subfield, default=None):
        """
        return values[idx].get(key) if existing
        'default' otherwise
        """
        try:
            return values[idx].get(subfield, default)
        except IndexError:
            return default

    def getViewFor(self, instance, idx, subfield, joinWith=', '):
        """
        formatted value of the subfield for display
        """
        raw = self.getRaw(instance)[idx].get(subfield,'')
        if type(raw) in (type(()), type([])):
            raw = joinWith.join(raw)
        # Prevent XSS attacks by quoting all user input
        raw = html_quote(str(raw))
        # this is now very specific
        if subfield == 'email':
            return self.hideEmail(raw,instance)
        if subfield == 'homepage':
            return '<a href="%s">%s</a>' % (raw, raw)
        return raw.strip()

    # store string type subfield values as unicode
    
    def _encode_strings(self, value, instance, **kwargs):
        new_value = []
        for entry in value:
            new_value.append(
                RecordField._encode_strings(self, entry, instance, **kwargs)
                )
        return new_value

    def _decode_strings(self, value, instance, **kwargs):
        new_value = []
        for entry in value:
            new_value.append(
                RecordField._decode_strings(self, entry, instance, **kwargs)
                )
        return new_value

    # convert the records to persistent dictionaries
    def _to_dict(self, value):
        if type(value)!= list or type(value)!=tuple:
            if type(value) == dict:
                value = [value]
            elif type(value) == str:
                try:
                    value = eval(value.replace('}\n', '},'))
                except:
                    pass
        return [RecordField._to_dict(self, entry) for entry in value]

    if HAS_VALIDATION_CHAIN:
        security.declarePublic('validate')
        def validate(self, value, instance, errors={}, **kwargs):
            """
            Validate passed-in value using all subfield validators.
            Return None if all validations pass; otherwise, return failed
            result returned by validator
            """
            name = self.getName()
            if errors and errors.has_key(name):
                return True

            result = None
            for record in value:
                result = RecordField.validate(self,
                                              record,
                                              instance,
                                              errors={},
                                              **kwargs
                                              )
                if result: return result
            return result

InitializeClass(RecordsField)

registerField(RecordsField,
              title="Records",
              description="Used for storing a list of records",
              )

registerPropertyType('subfields', 'lines', RecordsField)
registerPropertyType('required_subfields', 'lines', RecordField)
registerPropertyType('subfield_validators', 'mapping', RecordField)
registerPropertyType('subfield_types', 'mapping', RecordsField)
registerPropertyType('subfield_vocabularies', 'mapping', RecordsField)
registerPropertyType('subfield_labels', 'mapping', RecordsField)
registerPropertyType('subfield_sizes', 'mapping', RecordsField)
registerPropertyType('subfield_maxlength', 'mapping', RecordsField)
registerPropertyType('innerJoin', 'string', RecordsField)
registerPropertyType('fixedSize', 'boolean', RecordsField)
registerPropertyType('minimalSize', 'int', RecordsField)
registerPropertyType('maximalSize', 'int', RecordsField)



##########################################################################
#                                                                        #
#           copyright (c) 2005 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

""" demonstrates the use of the formattable name(s) field"""

from Products.Archetypes.public import *
from Products.ATExtensions.ateapi import *
from Products.ATExtensions.config import PROJECTNAME


schema = BaseSchema + Schema((
    FormattableNameField('name1'),
    FormattableNameField('name2',
                         subfields = ('title',
                                      'firstnames',
                                      'lastname',
                                      'homepage'),
                         widget = FormattableNameWidget(
    format="%f%l: %F %m. %L (%T)",
    join_abbr=".",
    withURL = True,
    ),
                         ),
    FormattableNamesField('friends',
                          widget = FormattableNamesWidget(format="%L %f%m"),
                          )
    ))

class FormattableNameDemo(BaseContent):
    """
    Demo from ATExtensions
    Illustrates useage of the formattable name(s) field
    """
    content_icon = "user.gif"
    schema = schema
    global_allow = 0

    _at_rename_after_creation = True

registerType(FormattableNameDemo, PROJECTNAME)



##########################################################################
#                                                                        #
#           copyright (c) 2004, 2005 ITB, Humboldt-University Berlin     #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

""" demonstrates the use of ATExtensions """

from Products.Archetypes.public import *
from Products.ATExtensions.ateapi import *
from Products.ATExtensions.config import PROJECTNAME


schema = BaseSchema + Schema((
    SimpleNameField('principal_investigator',
                    widget=RecordWidget(label='Principal Investigator',
                                        description='The person leading the group',
                                        ),
                    ),
    LabeledUrlField('home_institute',
                    required=0,
                    widget=LabeledUrlWidget(label='Institution',
                                            description='Name and Url of the institution hosting the group'),
                ),
    PhoneNumbersField('phone',
                      schemata="contact",
                      ),
    NetAddressField('e_address',
                    schemata="contact",
                    widget= RecordWidget(label='E-Contact',
                                         description='Electronic contact addresses'),
                    ),
    SmartEmailField('safe_email',
                    schemata="contact",
                    ),
    AddressField('street_address',
                 schemata="contact",
                 required=1,
                 widget=RecordWidget(label='Address',
                                     description='Street (or postal) contact address'),
                 ),
    # separate schemata page for members and partner groups;
    # the useage of minimalSize and maximalSize is for demonstration only
    MultipleNamesField('group_members',
                       schemata='members',
                       minimalSize = 1,  # effectively 2 because 'More' is enabled
                       maximalSize = 4,
                       widget=RecordsWidget(label="Group Members",
                                            description="Members of this group; press 'more' to enable additional entries (restricted to 4 at most)."),
                       ),
    # and a relation field for partner groups
    ReferenceField('partners',
                   schemata='members',
                   relationship="partner_groups",
                   multiValued=1,
                   allowed_types=("WorkingGroup",),
                   widget=ReferenceWidget(description="Groups with which this group has a collaborations",
                                          addable=1,
                                          destination="/",
                                          ),
                   ),
    # and here a record field configured in the schema declaration
    RecordsField('seminars',
                 schemata='seminars',
                 subfields = ('speaker', 'room', 'date'),
                 subfield_types = {'date':'datetime'},
                 subfield_sizes = {'speaker' : 15, 'room': 6},
                 innerJoin = ', ',
                 outerJoin = '<br />',
                 widget = RecordsWidget(
    helper_js=('jscalendar/calendar_stripped.js',
               'jscalendar/calendar-en.js'),
    helper_css=('jscalendar/calendar-system.css',),
            )

                ),
    TextField('short_description',
              widget=TextAreaWidget(label='Description',
                                    description="A short (plain text) description of the research group's focus.",
                                    ),
              accessor='getShortDescription',
              edit_accessor='getShortDescription',
              mutator='setShortDescription',
              ),
    StringField('test_combo',
                schemata='testing',
                vocabulary='DemoVocab',
                widget=ComboWidget(description="You need to select 'other' if you want to specify the value yourself below."),
                ),
    UrlField('url',
             schemata='testing',
             ),
    RemoteTextField('remote',
                    schemata='remote',
                    default_output_type='text/html',
                    allowable_content_types = ('text/plain',
                                               'text/html',
                                               'text/structured',
                                               ),
                    ),
    ))

class WorkingGroup(BaseContent):
    """
    Demo from ATExtensions
    stores information about a research group
    """
    content_icon = "group.gif"
    schema = schema
    _at_rename_after_creation = True

    # enable field sharing between 'short_description' and 'description'

    def getShortDescription(self, **kw):
        """ get the 'description' and put it in the 'short_description'
        """
        return self.Description()

    def setShortDescription(self, val, **kw):
        """ synchronize 'short_description' and 'description'
        """
        self.setDescription(val)

    # for the catalog metadata

    def Name(self):
        """
        the name of the group
        """
        return self.Title() + ', ' + self.getHome_institute()

    def LastName(self):
        """PI's last name; needed by the catalog for sorting"""
        return self.getPrincipal_investigator().get('last_name','')


    def PI(self):
        """
        the name of the principal investigator
        """
        pi_name = self.getPrincipal_investigator().get('first_name','') \
                  + ' ' + \
                  self.getPrincipal_investigator().get('last_name','')
        return pi_name.strip()

    def City(self):
        """ subfield of street address """
        return self.getStreet_address().get('city','')

    def Country(self):
        """ subfield of street address """
        return self.getStreet_address().get('country','')

    # the demo vocab

    def DemoVocab(self):
        """just to have some display list"""
        return getDisplayList(self, 'phone_number_types')


def modify_fti(fti):
    fti['allow_discussion'] = 1
    fti['global_allow'] = 0
    # hide unnecessary tabs (usability enhancement)
    for a in fti['actions']:
        if a['id'] in ('references',):
            a['visible'] = 0
    return fti

registerType(WorkingGroup, PROJECTNAME)



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerWidget
from records import RecordsWidget
from formattablename import FormattableNameWidget

class FormattableNamesWidget(RecordsWidget, FormattableNameWidget):
    _properties = RecordsWidget._properties.copy()
    _properties.update({
        'macro_view' : "formattablename_view",
        'format' : '%T %F %M %P %L %S',
        'formatter' : {},
        'abbr' : {},
        'join_abbr' : '',
        'withURL' : False,
        'sep' : '',
        'lastsep' : '',
        'empty_marker' : '',
        })
    _format_properties = ['format', 'formatter', 'abbr', 'join_abbr',
                          'withURL', 'sep', 'lastsep', 'empty_marker']
    security = ClassSecurityInfo()


InitializeClass(FormattableNamesWidget)

registerWidget(FormattableNamesWidget,
               title='FormattableNames',
               description="Renders formattable names.",
               used_for=('Products.ATExtensions.fields.FormattableNamesField',)
               )



from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Field import StringField
from Products.Archetypes.Registry import registerField

from Products.ATExtensions.widget import EmailWidget

class EmailField(StringField):
    """specific field for emails""" 
    _properties = StringField._properties.copy()
    _properties.update({
        'type' : 'email',
        'validators' : ('isEmail'),
        'widget' : EmailWidget,
        })
    security = ClassSecurityInfo()

InitializeClass(EmailField)

registerField(EmailField,
              title="Email",
              description="Used for storing a validated email.",
              )



##########################################################################
#                                                                        #
#           copyright (c) 2004 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

""" utilities for the ATExtensions product """

from Products.CMFCore.utils import getToolByName
from Products.Archetypes.public import DisplayList

def getValues(self, prop_name):
    ptool = getToolByName(self, 'portal_properties', None)
    if ptool and hasattr(ptool, 'extensions_properties'):
        return ptool.extensions_properties.getProperty(prop_name, None)
    else:
        return None

def makeDisplayList(values=None,add_select=True):
    if values and type(values) not in [type([]), type(())]:
        values = (values,)
    if not values: values = []
    if add_select:
        results = [['', 'Select'],]
    else:
        results = []
    for x in values:
        results.append([x,x])
    values_tuple = tuple(results)
    return DisplayList(values_tuple)

def getDisplayList(self, prop_name=None,add_select=True):
    return makeDisplayList(getValues(self, prop_name),add_select)



# an 'import_various' step to deal with form controller settings
# as there seems to be no support for doing this via GS at the moment
# (March 12, 2007)

def importVarious(context):
    """ Import various settings for ATExtensions

    This provisional handler will be removed again as soon as
    full handlers are implemented for this step.
    """
    site = context.getSite()
    pfc = site.portal_form_controller
    pfc.addFormValidators('base_edit',
                          '',   # context_type
                          'more',
                          '')   # validators
    pfc.addFormAction('base_edit',
                      'success',
                      '',    # context_type
                      'more',
                      'traverse_to',
                      'string:more_edit')
    pfc.addFormValidators('atct_edit',
                          '',   # context_type
                          'more',
                          '')   # validators
    pfc.addFormAction('atct_edit',
                      'success',
                      '',    # context_type
                      'more',
                      'traverse_to',
                      'string:more_edit')
    return "Added validator and action for the 'more' button to " \
           "the form controller."



from Acquisition import aq_get
from zope.i18n import translate
from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFCore.utils import getToolByName
from Products.Archetypes.Field import StringField
from Products.Archetypes.Registry import registerField
from Products.Archetypes.Registry import registerPropertyType

from Products.ATExtensions.widget import CommentWidget

class CommentField(StringField):
    """specific field for URLs""" 
    _properties = StringField._properties.copy()
    _properties.update({
        'type' : 'comment',
        'comment' : ' ',
        'comment_type' : 'text/structured',
        'comment_msgid' : '',
        'comment_method' : None,
        'widget' : CommentWidget,
        })
    security = ClassSecurityInfo()

    def get(self, instance, **kwargs):
        domain = self.widget.i18n_domain
        request = aq_get(instance, 'REQUEST', None)
        if self.comment_method is not None:
            self.comment = getattr(instance, self.comment_method)()
        if request is not None:
            if self.comment_msgid:
                comment = translate(domain=domain, msgid=self.comment_msgid, default=self.comment, context=request)
            else:
                comment = translate(domain=domain, msgid=self.comment, default=self.comment, context=request)    
        else:
            comment = self.comment
        transforms = getToolByName(instance, 'portal_transforms', None)
        if transforms is None:
            return comment
        return transforms.convertTo('text/html',
                                    comment,
                                    context=instance,
                                    mimetype=self.comment_type).getData()

    def set(self, instance, value, **kwargs):
        pass

InitializeClass(CommentField)

registerField(CommentField,
              title="Comment",
              description="Used for inserting comments into the views",
              )
registerPropertyType('comment', 'string', CommentField)
registerPropertyType('comment_type', 'string', CommentField)



## Script (Python) "emailMask"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters= emailaddress
##title=
##
# This is the script which is used to mask the emailaddresses
# in smartemailwidgets.
#

return emailaddress.replace('@',' at ')


import os, urllib
from DateTime.DateTime import DateTime
from Products.Archetypes.public import TextField, DisplayList
from Products.Archetypes.Registry import registerField
from Products.Archetypes.Registry import registerPropertyType
from Products.Archetypes.annotations import AT_ANN_STORAGE as KEY
from Products.Archetypes.annotations import getAnnotation

from Products.ATExtensions.widget.remotetext import RemoteTextWidget

def _pipe(command, input=''):
    # XXX TODO  what kind of pipe do we want?
    fi, fo, fe = os.popen3(command, 't')
    fi.write(input)
    fi.close()
    output = fo.read()
    fo.close()
    error = fe.read()
    fe.close()
    return output, error
    

class RemoteTextField(TextField):
    """Field to represent text coming from a remote source like
    another web site (url) or a subversion repository"""

    _properties = TextField._properties.copy()
    _properties.update({
        'type' : 'remote_text',
        'source_types' : ('svn', 'web'),
        'timeout' : 1, # should be one day
        'customized' : False,
        'widget' : RemoteTextWidget,
        })

    def getSourceTypeVocabulary(self):
        """All supported repository types """
        d = DisplayList()
        for t in self.source_types:
            d.add(t,t)
        return d

    # source handling
    def getSourcePath(self, instance):
        """the source path from the instance annotations or empty string"""
        ann = getAnnotation(instance)
        subkey = "%s-source-path" % self.getName()
        return ann.getSubkey(KEY, subkey, default='')

    def setSourcePath(self, instance, value):
        """Stores source path in the instance annotations"""
        ann = getAnnotation(instance)
        subkey = "%s-source-path" % self.getName()
        return ann.setSubkey(KEY, value, subkey)

    def getSourceType(self, instance):
        """the source path from the instance annotations or empty string"""
        ann = getAnnotation(instance)
        subkey = "%s-source-type" % self.getName()
        return ann.getSubkey(KEY, subkey, default='')

    def setSourceType(self, instance, value):
        """Stores source path in the instance annotations"""
        ann = getAnnotation(instance)
        subkey = "%s-source-type" % self.getName()
        return ann.setSubkey(KEY, value, subkey)

    def getTimeout(self, instance):
        """the timeout from the instance annotations or empty string"""
        ann = getAnnotation(instance)
        subkey = "%s-timeout" % self.getName()
        return ann.getSubkey(KEY, subkey, default=self.timeout)

    def setTimeout(self, instance, value):
        """Stores the timeout in the instance annotations"""
        ann = getAnnotation(instance)
        subkey = "%s-timeout" % self.getName()
        return ann.setSubkey(KEY, value, subkey)

    def getLastUpdate(self, instance):
        """time of the last update or None"""
        ann = getAnnotation(instance)
        subkey = "%s-last-update" % self.getName()
        return ann.getSubkey(KEY, subkey, default=None)

    def setLastUpdate(self, instance, value):
        """Stores the time of the last update in the instance annotations"""
        ann = getAnnotation(instance)
        subkey = "%s-last-update" % self.getName()
        return ann.setSubkey(KEY, value, subkey)

    def is_customized(self, instance):
        """Flag whether this field has been customized"""
        ann = getAnnotation(instance)
        subkey = "%s-customized" % self.getName()
        return ann.getSubkey(KEY, subkey, default=self.customized)

    def customize(self, instance):
        """Set the customize flag to True"""
        ann = getAnnotation(instance)
        subkey = "%s-customized" % self.getName()
        ann.setSubkey(KEY, True, subkey)
        return None
        
    def uncustomize(self, instance):
        """Set the customize flag to False"""
        ann = getAnnotation(instance)
        subkey = "%s-customized" % self.getName()
        ann.setSubkey(KEY, False, subkey)
        return None

    def get(self, instance, **kw):
        if self._isexpired(instance):
            self.set(instance)
        return TextField.get(self, instance, **kw)

    def _isexpired(self, instance):
        if self.is_customized(instance):
            return False
        last_update = self.getLastUpdate(instance)
        if last_update is None:
            return False  # not yet loaded -> not expired
        timeout = self.getTimeout(instance)
        now = DateTime()
        if (now - timeout) > last_update:
            return True
        return False

    def set(self, instance, value=None, **kw):
        source_text = None
        if value is None or not self.is_customized(instance):
            source_text = self.getSourceText(instance, **kw)
        if source_text:
            value = source_text
            now = DateTime()
            self.setLastUpdate(instance, now)
        TextField.set(self, instance, value, **kw)

    def getSourceText(self, instance, **kw):
        """main method to fetch the value from the remote site"""
        stype = self.getSourceType(instance)
        value = ''
        if stype == 'web':
            value = self.getTextFromUrl(instance)
        if stype == 'svn':
            value = self.getTextFromSvn(instance)
        return value or ''

    def getTextFromUrl(self, instance):
        source_path = self.getSourcePath(instance)
        try:
            # better use Tiran's 'urlupload'?
            value = urllib.urlopen(source_path).read()
        except:  # XXX FIXME what to catch here?
            value = ''

        return value

    def getTextFromSvn(self, instance):
        source_path = self.getSourcePath(instance)
        error = None
        command = "svn export %s" % source_path
        try:
            value, error = _pipe(command)
        except:  # XXX FIXME what to catch here? - or is pipe
            # robust enough never to blow up??
            value = ''

        if error:
            return ''
        return value

registerField(RemoteTextField,
              title="Remote Text Field",
              description="Populate a field from a remote source")

registerPropertyType('source_types', 'lines', RemoteTextField)
registerPropertyType('timeout', 'DateTime', RemoteTextField)
registerPropertyType('customized', 'boolean', RemoteTextField)




##########################################################################
#                                                                        #
#           copyright (c) 2004 ITB, Humboldt-University Berlin           #
#           written by: Raphael Ritz, r.ritz@biologie.hu-berlin.de       #
#                                                                        #
##########################################################################

"""plone installer script"""

from StringIO import StringIO

from Products.CMFCore.utils import getToolByName
from Products.CMFFormController.FormAction import FormActionKey
from Products.CMFFormController.FormValidator import FormValidatorKey

from Products.Archetypes.public import listTypes
from Products.Archetypes.Extensions.utils \
     import installTypes, install_subskin

from Products.ATExtensions.config import *


def setupProperties(self, out):
    ptool = getToolByName(self, 'portal_properties')
    psheet = getattr(ptool, 'extensions_properties', None)

    if not psheet:
        ptool.addPropertySheet('extensions_properties',
                               'Properties of the ATExtension product')

        ps = getattr(ptool, 'extensions_properties')
        ps._properties = ps._properties + (
            {'id':'phone_number_types', 'type':'lines', 'mode':'wd'},
            {'id':'country_names', 'type':'lines', 'mode':'wd'},
            )
        ps._updateProperty('phone_number_types', PHONE_NUMBER_TYPES)
        ps._updateProperty('country_names', COUNTRY_NAMES)
        out.write("Added %s's properties." % PROJECTNAME)
    else:
        out.write("Found %s's properties." % PROJECTNAME)

def configureFormController(self, out):
    pfc = getToolByName(self, 'portal_form_controller')
    pfc.addFormValidators('base_edit',
                          '',   # context_type
                          'more',
                          '')   # validators
    pfc.addFormAction('base_edit',
                      'success',
                      '',    # context_type
                      'more',
                      'traverse_to',
                      'string:more_edit')
    out.write("Added validator and action for the 'more' button to the form controller.")



def install(self):
    out = StringIO()
    if INSTALL_DEMO_TYPES:
        installTypes(self, out, listTypes(PROJECTNAME), PROJECTNAME)
    install_subskin(self, out, GLOBALS)
    setupProperties(self, out)
    configureFormController(self, out)
    out.write("Successfully installed %s." % PROJECTNAME)
    return out.getvalue()

# for the uninstall we need to take care of the form controller
# explicitly

def reconfigureFormController(self, out):
    pfc = getToolByName(self, 'portal_form_controller')

    #BAAH no Python API for deleting actions or validators in FormController
    #lets get our hands dirty
    container = pfc.actions
    try:
        container.delete(FormActionKey('base_edit', 'success', '',
                                       'more', pfc))
    except KeyError: pass

    container = pfc.validators
    try:
        container.delete(FormValidatorKey('base_edit', '',
                                          'more', pfc))
    except KeyError: pass

    out.write("Removed validator and action for the 'more' button from the form controller.")

def uninstall(self):
    out = StringIO()
    reconfigureFormController(self, out)
    # all the rest we leave to the quick installer
    out.write("Uninstalled %s." % PROJECTNAME)
    return out.getvalue()



from setuptools import setup, find_packages

version = '1.1'

setup(name='Products.ATExtensions',
      version=version,
      description="This package provides some further fields and widgets for Archetypes.",
      long_description=open("README.txt").read() + "\n" +
                       open("CHANGES.txt").read(),
      classifiers=[
        "Framework :: Plone",
        "Programming Language :: Python",
        ],
      keywords='',
      author='Raphael Ritz',
      author_email='raphael.ritz@gmail.com',
      url='http://pypi.python.org/pypi/Products.ATExtensions',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
      ],
      )



# just to be compatible with the general AT approach

from Products.CMFCore.permissions import AddPortalContent

ADD_CONTENT_PERMISSION = AddPortalContent
PROJECTNAME = "ATExtensions"
SKINS_DIR = 'skins'
GLOBALS = globals()

INSTALL_DEMO_TYPES = True
INSTALL_DEMO_TYPES = False

try:
    from Products.validation import ValidationChain
    HAS_VALIDATION_CHAIN = 1
except ImportError:
    HAS_VALIDATION_CHAIN = 0

PHONE_NUMBER_TYPES = [
    "Office",
    "Secretariat",
    "Laboratory",
    "Mobile",
    "Fax",
    "Private",
    ]

COUNTRY_NAMES = [
    "Afghanistan",
    "Albania",
    "Algeria",
    "American Samoa",
    "Andorra",
    "Angola",
    "Anguilla",
    "Antarctica",
    "Antigua Barbuda",
    "Argentina",
    "Armenia",
    "Aruba",
    "Australia",
    "Austria",
    "Azerbaijan",
    "Bahamas",
    "Bahrain",
    "Bangladesh",
    "Barbados",
    "Belarus",
    "Belgium",
    "Belize",
    "Benin",
    "Bermuda",
    "Bhutan",
    "Bolivia",
    "Bosnia Herzegovina",
    "Botswana",
    "Bouvet Island",
    "Brazil",
    "British Indian Ocean Territory",
    "Brunei Darussalam",
    "Bulgaria",
    "Burkina Faso",
    "Burundi",
    "Cambodia",
    "Cameroon",
    "Canada",
    "Cape Verde",
    "Cayman Islands",
    "Central African Republic",
    "Chad",
    "Chile",
    "China",
    "Christmas Island",
    "Cocos (Keeling) Islands",
    "Colombia",
    "Comoros",
    "Congo",
    "Congo, Democratic Republic",
    "Cook Islands",
    "Costa Rica",
    "Cote D'Ivoire",
    "Croatia",
    "Cuba",
    "Cyprus",
    "Czech Republic",
    "Denmark",
    "Djibouti",
    "Dominica",
    "Dominican Republic",
    "Ecuador",
    "Egypt",
    "El Salvador",
    "Equatorial Guinea",
    "Eritrea",
    "Estonia",
    "Ethiopia",
    "Falkland Islands (Malvinas)",
    "Faroe Islands",
    "Fiji",
    "Finland",
    "France",
    "French Guiana",
    "French Polynesia",
    "French Southern Territories",
    "Gabon",
    "Gambia",
    "Georgia",
    "Germany",
    "Ghana",
    "Gibraltar",
    "Greece",
    "Greenland",
    "Grenada",
    "Guadeloupe",
    "Guam",
    "Guatemala",
    "Guinea",
    "Guinea-Bissau",
    "Guyana",
    "Haiti",
    "Heard Island Mcdonald Islands",
    "Holy See (Vatican City State)",
    "Honduras",
    "Hong Kong",
    "Hungary",
    "Iceland",
    "India",
    "Indonesia",
    "Iran, Islamic Republic",
    "Iraq",
    "Ireland",
    "Israel",
    "Italy",
    "Jamaica",
    "Japan",
    "Jordan",
    "Kazakhstan",
    "Kenya",
    "Kiribati",
    "Korea, Democratic People's Republic",
    "Korea, Republic",
    "Kuwait",
    "Kyrgyzstan",
    "Lao People's Democratic Republic",
    "Latvia",
    "Lebanon",
    "Lesotho",
    "Liberia",
    "Libyan Arab Jamahiriya",
    "Liechtenstein",
    "Lithuania",
    "Luxembourg",
    "Macao",
    "Macedonia, Former Yugoslav Republic",
    "Madagascar",
    "Malawi",
    "Malaysia",
    "Maldives",
    "Mali",
    "Malta",
    "Marshall Islands",
    "Martinique",
    "Mauritania",
    "Mauritius",
    "Mayotte",
    "Mexico",
    "Micronesia, Federated States",
    "Moldova, Republic",
    "Monaco",
    "Mongolia",
    "Montserrat",
    "Morocco",
    "Mozambique",
    "Myanmar",
    "Namibia",
    "Nauru",
    "Nepal",
    "Netherlands",
    "Netherlands Antilles",
    "New Caledonia",
    "New Zealand",
    "Nicaragua",
    "Niger",
    "Nigeria",
    "Niue",
    "Norfolk Island",
    "Northern Mariana Islands",
    "Norway",
    "Oman",
    "Pakistan",
    "Palau",
    "Palestinian Territory, Occupied",
    "Panama",
    "Papua New Guinea",
    "Paraguay",
    "Peru",
    "Philippines",
    "Pitcairn",
    "Poland",
    "Portugal",
    "Puerto Rico",
    "Qatar",
    "Reunion",
    "Romania",
    "Russian Federation",
    "Rwanda",
    "Saint Helena",
    "Saint Kitts Nevis",
    "Saint Lucia",
    "Saint Pierre Miquelon",
    "Saint Vincent Grenadines",
    "Samoa",
    "San Marino",
    "Sao Tome Principe",
    "Saudi Arabia",
    "Senegal",
    "Serbia Montenegro",
    "Seychelles",
    "Sierra Leone",
    "Singapore",
    "Slovakia",
    "Slovenia",
    "Solomon Islands",
    "Somalia",
    "South Africa",
    "South Georgia South Sandwich Islands",
    "Spain",
    "Sri Lanka",
    "Sudan",
    "Suriname",
    "Svalbard Jan Mayen",
    "Swaziland",
    "Sweden",
    "Switzerland",
    "Syrian Arab Republic",
    "Taiwan, Province China",
    "Tajikistan",
    "Tanzania, United Republic",
    "Thailand",
    "Timor-Leste",
    "Togo",
    "Tokelau",
    "Tonga",
    "Trinidad Tobago",
    "Tunisia",
    "Turkey",
    "Turkmenistan",
    "Turks Caicos Islands",
    "Tuvalu",
    "Uganda",
    "Ukraine",
    "United Arab Emirates",
    "United Kingdom",
    "United States",
    "United States Minor Outlying Islands",
    "Uruguay",
    "Uzbekistan",
    "Vanuatu",
    "Venezuela",
    "Vietnam",
    "Virgin Islands, British",
    "Virgin Islands, U.S.",
    "Wallis Futuna",
    "Western Sahara",
    "Yemen",
    "Zambia",
    "Zimbabwe",
    ]




from App.class_init import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.Archetypes.Registry import registerWidget
from record import RecordWidget

class FormattableNameWidget(RecordWidget):
    _properties = RecordWidget._properties.copy()
    _properties.update({
        'macro_view' : "formattablename_view",
        'format' : '%T %F %M %P %L %S',
        'formatter' : {},
        'abbr' : {},
        'join_abbr' : '',
        'withURL' : False,
        })
    _format_properties = ['format', 'formatter', 'abbr',
                          'join_abbr', 'withURL']
    
    security = ClassSecurityInfo()

    security.declarePublic('getKwArgs')
    def getKwArgs(self):
        """mapping of all non-empty formatting properties"""
        result = {}
        for property in self._format_properties:
            value = getattr(self, property, None)
            if value:
                result[property] = value
        return result

InitializeClass(FormattableNameWidget)

registerWidget(FormattableNameWidget,
               title='FormattableName',
               description="Renders a formattable name.",
               used_for=('Products.ATExtensions.fields.FormattableNameField',)
               )



import re

from zope.interface import implements

from Products.validation.interfaces.IValidator import IValidator
from Products.validation.interfaces import ivalidator

try:
    from Products.CMFPlone.factory import _IMREALLYPLONE4 as PLONE4
    PLONE4 # pyflakes
except ImportError:
    PLONE4 = False


class PartialUrlValidator:

    if PLONE4:
        implements(IValidator)
    else:
        __implements__ = (ivalidator,)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        # field gets passed in via kwargs
        if '://' not in value:
            widget = kwargs.get('field').widget
            default_p = getattr(widget,'default_protocol','http')
            value = default_p + '://' + value
        pattern = re.compile(r'(ht|f)tps?://[^\s\r\n]+')
        m = pattern.match(value)
        if not m:
            return ("Validation failed(%s): value is %s"%(self.name,
                repr(value)))
        return 1
